@@ -1,7 +1,10 @@
+import contextlib
 import sys
 from collections import defaultdict
 from typing import DefaultDict
 
+from drf_spectacular.settings import spectacular_settings
+
 
 class GeneratorStats:
     _warn_cache: DefaultDict[str, int] = defaultdict(int)
@@ -15,8 +18,10 @@ class GeneratorStats:
         self._error_cache.clear()
 
     def emit(self, msg, severity):
+        if spectacular_settings.DISABLE_ERRORS_AND_WARNINGS:
+            return
         assert severity in ['warning', 'error']
-        msg = str(msg)
+        msg = _get_current_trace() + str(msg)
         cache = self._warn_cache if severity == 'warning' else self._error_cache
         if msg not in cache:
             print(f'{severity.capitalize()} #{len(cache)}: {msg}', file=sys.stderr)
@@ -47,6 +52,23 @@ def reset_generator_stats():
     GENERATOR_STATS.reset()
 
 
+_TRACES = []
+
+
+@contextlib.contextmanager
+def add_trace_message(trace_message):
+    """
+    Adds a message to be used as a prefix when emitting warnings and errors.
+    """
+    _TRACES.append(trace_message)
+    yield
+    _TRACES.pop()
+
+
+def _get_current_trace():
+    return ''.join(f"{trace}: " for trace in _TRACES if trace)
+
+
 def has_override(obj, prop):
     if not hasattr(obj, '_spectacular_annotation'):
         return False
