@@ -1,14 +1,13 @@
-from collections import OrderedDict
+from decimal import Decimal
 
 import yaml
 from rest_framework.exceptions import ErrorDetail
-from rest_framework.renderers import JSONRenderer, BaseRenderer
+from rest_framework.renderers import BaseRenderer, JSONRenderer
 
 
 class OpenApiYamlRenderer(BaseRenderer):
     media_type = 'application/vnd.oai.openapi'
-    charset = None
-    format = 'openapi'
+    format = 'yaml'
 
     def render(self, data, media_type=None, renderer_context=None):
         # disable yaml advanced feature 'alias' for clean, portable, and readable output
@@ -26,14 +25,20 @@ class OpenApiYamlRenderer(BaseRenderer):
             return scalar
         Dumper.add_representer(str, multiline_str_representer)
 
-        def ordered_dict_representer(dumper, data):
-            return dumper.represent_dict(data)
-        Dumper.add_representer(OrderedDict, ordered_dict_representer)
+        def decimal_representer(dumper, data):
+            # prevent emitting "!! float" tags on fractionless decimals
+            value = str(data)
+            if '.' in value:
+                return dumper.represent_scalar('tag:yaml.org,2002:float', value)
+            else:
+                return dumper.represent_scalar('tag:yaml.org,2002:int', value)
+        Dumper.add_representer(Decimal, decimal_representer)
 
         return yaml.dump(
             data,
             default_flow_style=False,
             sort_keys=False,
+            allow_unicode=True,
             Dumper=Dumper
         ).encode('utf-8')
 
