@@ -8,6 +8,7 @@ from django.utils.translation import gettext_lazy as _
 from rest_framework.renderers import TemplateHTMLRenderer
 from rest_framework.response import Response
 from rest_framework.reverse import reverse
+from rest_framework.settings import api_settings
 from rest_framework.views import APIView
 
 from drf_spectacular.plumbing import get_relative_url, set_query_parameters
@@ -30,6 +31,11 @@ if spectacular_settings.SERVE_INCLUDE_SCHEMA:
 else:
     SCHEMA_KWARGS = {'exclude': True}
 
+if spectacular_settings.SERVE_AUTHENTICATION is not None:
+    AUTHENTICATION_CLASSES = spectacular_settings.SERVE_AUTHENTICATION
+else:
+    AUTHENTICATION_CLASSES = api_settings.DEFAULT_AUTHENTICATION_CLASSES
+
 
 class SpectacularAPIView(APIView):
     __doc__ = _("""
@@ -42,7 +48,7 @@ class SpectacularAPIView(APIView):
         OpenApiYamlRenderer, OpenApiYamlRenderer2, OpenApiJsonRenderer, OpenApiJsonRenderer2
     ]
     permission_classes = spectacular_settings.SERVE_PERMISSIONS
-
+    authentication_classes = AUTHENTICATION_CLASSES
     generator_class = spectacular_settings.DEFAULT_GENERATOR_CLASS
     serve_public = spectacular_settings.SERVE_PUBLIC
     urlconf = spectacular_settings.SERVE_URLCONF
@@ -76,11 +82,37 @@ class SpectacularJSONAPIView(SpectacularAPIView):
 class SpectacularSwaggerView(APIView):
     renderer_classes = [TemplateHTMLRenderer]
     permission_classes = spectacular_settings.SERVE_PERMISSIONS
+    authentication_classes = AUTHENTICATION_CLASSES
     url_name = 'schema'
     url = None
     template_name = 'drf_spectacular/swagger_ui.html'
     template_name_js = 'drf_spectacular/swagger_ui.js'
 
+    @extend_schema(exclude=True)
+    def get(self, request, *args, **kwargs):
+        schema_url = self.url or get_relative_url(reverse(self.url_name, request=request))
+        return Response(
+            data={
+                'dist': spectacular_settings.SWAGGER_UI_DIST,
+                'favicon_href': spectacular_settings.SWAGGER_UI_FAVICON_HREF,
+                'schema_url': set_query_parameters(
+                    url=schema_url,
+                    lang=request.GET.get('lang')
+                ),
+                'settings': json.dumps(spectacular_settings.SWAGGER_UI_SETTINGS),
+                'template_name_js': self.template_name_js
+            },
+            template_name=self.template_name,
+        )
+
+
+class SpectacularSwaggerSplitView(SpectacularSwaggerView):
+    """
+    Alternate Swagger UI implementation that separates the html request from the
+    javascript request to cater to web servers with stricter CSP policies.
+    """
+    url_self = None
+
     @extend_schema(exclude=True)
     def get(self, request, *args, **kwargs):
         if request.GET.get('script') is not None:
@@ -97,7 +129,7 @@ class SpectacularSwaggerView(APIView):
                 content_type='application/javascript',
             )
         else:
-            script_url = request.get_full_path()
+            script_url = self.url_self or request.get_full_path()
             return Response(
                 data={
                     'dist': spectacular_settings.SWAGGER_UI_DIST,
@@ -115,6 +147,7 @@ class SpectacularSwaggerView(APIView):
 class SpectacularRedocView(APIView):
     renderer_classes = [TemplateHTMLRenderer]
     permission_classes = spectacular_settings.SERVE_PERMISSIONS
+    authentication_classes = AUTHENTICATION_CLASSES
     url_name = 'schema'
     url = None
     template_name = 'drf_spectacular/redoc.html'
