@@ -1,3 +1,4 @@
+from typing import Union
 from unittest import mock
 
 import pytest
@@ -13,6 +14,7 @@ from drf_spectacular.generators import SchemaGenerator
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import PolymorphicProxySerializer, extend_schema, extend_schema_view
 from tests import generate_schema
+from tests.models import SimpleModel, SimpleSerializer
 
 
 def test_serializer_name_reuse(capsys):
@@ -78,26 +80,22 @@ def test_no_queryset_warn(capsys):
     generate_schema('x1', X1Viewset)
     stderr = capsys.readouterr().err
     assert 'obtaining queryset from' in stderr  # warning 1
-    assert 'failed to obtain model through view\'s queryset' in stderr  # warning 2
+    assert 'X1Viewset: failed to obtain model through view\'s queryset' in stderr  # warning 2
 
 
 def test_path_param_not_in_model(capsys):
-    class M3(models.Model):
-        pass  # pragma: no cover
-
-    class XSerializer(serializers.Serializer):
-        uuid = serializers.UUIDField()
-
     class XViewset(viewsets.ReadOnlyModelViewSet):
-        serializer_class = XSerializer
-        queryset = M3.objects.none()
+        serializer_class = SimpleSerializer
+        queryset = SimpleModel.objects.none()
 
         @action(detail=True, url_path='meta/(?P<ephemeral>[^/.]+)', methods=['POST'])
         def meta_param(self, request, ephemeral, pk):
             pass  # pragma: no cover
 
     generate_schema('x1', XViewset)
-    assert 'no such field' in capsys.readouterr().err
+    stderr = capsys.readouterr().err
+    assert 'no such field' in stderr
+    assert 'XViewset' in stderr
 
 
 def test_no_authentication_scheme_registered(capsys):
@@ -112,7 +110,9 @@ def test_no_authentication_scheme_registered(capsys):
         authentication_classes = [XAuth]
 
     generate_schema('x', XViewset)
-    assert 'no OpenApiAuthenticationExtension registered' in capsys.readouterr().err
+    stderr = capsys.readouterr().err
+    assert 'no OpenApiAuthenticationExtension registered' in stderr
+    assert 'XViewset' in stderr
 
 
 def test_serializer_not_found(capsys):
@@ -120,7 +120,7 @@ def test_serializer_not_found(capsys):
         pass  # pragma: no cover
 
     generate_schema('x', XViewset)
-    assert 'Exception raised while getting serializer' in capsys.readouterr().err
+    assert 'XViewset: exception raised while getting serializer.' in capsys.readouterr().err
 
 
 def test_extend_schema_unknown_class(capsys):
@@ -155,7 +155,7 @@ def test_no_serializer_class_on_apiview(capsys):
             pass  # pragma: no cover
 
     generate_schema('x', view=XView)
-    assert 'Unable to guess serializer for' in capsys.readouterr().err
+    assert 'XView: unable to guess serializer.' in capsys.readouterr().err
 
 
 def test_unable_to_follow_field_source_through_intermediate_property_warning(capsys):
@@ -178,7 +178,7 @@ def test_unable_to_follow_field_source_through_intermediate_property_warning(cap
 
     generate_schema('x', view=XAPIView)
     assert (
-        'could not follow field source through intermediate property'
+        'XAPIView: XSerializer: could not follow field source through intermediate property'
     ) in capsys.readouterr().err
 
 
@@ -206,8 +206,39 @@ def test_unable_to_derive_function_type_warning(capsys):
 
     generate_schema('x', view=XAPIView)
     stderr = capsys.readouterr().err
-    assert 'unable to resolve type hint for function "x"' in stderr
-    assert 'unable to resolve type hint for function "get_y"' in stderr
+    assert 'XAPIView: XSerializer: unable to resolve type hint for function "x"' in stderr
+    assert 'XAPIView: XSerializer: unable to resolve type hint for function "get_y"' in stderr
+
+
+def test_unable_to_traverse_union_type_hint(capsys):
+    class Foo:
+        foo_value: int = 1
+
+    class Bar:
+        pass
+
+    class FailingFieldSourceTraversalModel3(models.Model):
+        @property
+        def foo_or_bar(self) -> Union[Foo, Bar]:
+            pass  # pragma: no cover
+
+    class XSerializer(serializers.ModelSerializer):
+        foo_value = serializers.ReadOnlyField(source='foo_or_bar.foo_value')
+
+        class Meta:
+            model = FailingFieldSourceTraversalModel3
+            fields = '__all__'
+
+    class XAPIView(APIView):
+        @extend_schema(responses=XSerializer)
+        def get(self, request):
+            pass  # pragma: no cover
+
+    generate_schema('foo_value', view=XAPIView)
+    stderr = capsys.readouterr().err
+    assert 'could not traverse Union type' in stderr
+    assert 'Foo' in stderr
+    assert 'Bar' in stderr
 
 
 def test_operation_id_collision_resolution(capsys):
@@ -282,3 +313,17 @@ def test_warning_operation_id_on_extend_schema_view(capsys):
     generate_schema('x', view=XAPIView)
     stderr = capsys.readouterr().err
     assert 'using @extend_schema on viewset class XAPIView with parameters' in stderr
+
+
+def test_warning_request_body_not_resolvable(capsys):
+    class Dummy:
+        pass
+
+    @extend_schema(request=Dummy)
+    @api_view(['POST'])
+    def view_func(request, format=None):
+        pass  # pragma: no cover
+
+    generate_schema('x', view_function=view_func)
+    stderr = capsys.readouterr().err
+    assert 'could not resolve request body for POST /x' in stderr
